/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT. 
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRENTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 * $LastChangedRevision: 5679 $
 */

/** @file
 * @brief Enhanced ShockBurst Primary Transmitter example
 * @defgroup esb_ptx_example Enhanced ShockBurst Primary Transmitter (PTX) example
 * @{
 * @ingroup nrf_examples
 * 
 * @brief This example sends packets continuously. The contents of P0 is 
 * sent in the first payload byte (byte 0). 
 *
 * The example shows the minimum required setup for transmitting packets to a 
 * primary receiver (PRX) device.  
 * 
 * The following default radio parameters are being used:
 * - RF channel 2
 * - 2 Mbps data rate
 * - TX address 0xE7E7E7E7E7 
 * - 1 byte CRC  
 * 
 * The project @ref esb_prx_example can be used as a counterpart for receiving the data.
 * 
*/

#include <Nordic\reg24le1.h>
#include <stdint.h>
#include <stdbool.h>
#include "hal_nrf.h"
#include "hal_nrf_hw.h"
#include "lib_crypt.h"

#define total_size 16
#define data_size 11

// AES variables
idata uint8_t aes_key[16] = { 3,7,2,9,0,1,5,8,3,6,9,1,3,8,9,3 }; // Secret AES key
idata uint8_t counter[16] = { 0,0,0,0,0,1,2,3,4,5,6,7,8,9,0,1 }; // The 5LSB of the counter changes and should be transferred between PTX and PRX along with each payload

// Buffer variables
uint8_t xdata payload[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
uint8_t xdata encrypt[16] = {1,2,3,4,5,6,7,8,9,10,11};


// Global variables
bool radio_busy;

void main()
{
  uint8_t i;
	P0DIR = 0xFF;
	// Enable the radio clock
  RFCKEN = 1;
  // Enable RF interrupt
  RF = 1; 
  // Enable global interrupt
  EA = 1;
  // Power up radio
  hal_nrf_set_power_mode(HAL_NRF_PWR_UP);
	// Init AES crypt
	lib_crypt_init(aes_key, counter);
  while(1)
  {
		// P0 button press into first byte.
		encrypt[0] = ~P0;
		
		// Generate the 5LSB of the counter, the 5LSB must be exchanged between PTX and PRX along with payload, i.e RF packet contains payload + 5LSB
    // All data are encrypted with a 16byte AES key and a 16byte counter value, where the last 5LSB of the counter must be exchanged
    lib_crypt_generate_ls5b(counter);
		
		// Put counter into payload, PRX cannot decrypt without this.
		for(i=11; i <16; i++)
			payload[i] = counter[i-data_size];
		
		// Encrypt data, payload will be output buffer
		lib_crypt(payload,encrypt,data_size,counter);
    
		// Write payload to radio TX FIFO
    hal_nrf_write_tx_payload(payload, total_size);
    
		// Toggle radio CE signal to start transmission 
    CE_PULSE();
    radio_busy = true;

    // Wait for radio operation to finish
    while(radio_busy)
    ;
  }                                           
} 

// Radio interrupt
void rf_irq() interrupt INTERRUPT_RFIRQ
{
  uint8_t irq_flags;

  // Read and clear IRQ flags from radio
  irq_flags = hal_nrf_get_clear_irq_flags(); 

  switch(irq_flags)
  {
    // Transmission success
    case (1 << HAL_NRF_TX_DS):
      radio_busy = false;
      // Data has been sent
      break;
    // Transmission failed (maximum re-transmits)
    case (1 << HAL_NRF_MAX_RT): 
      hal_nrf_flush_tx();
      radio_busy = false;
      break;
  }
}